import logging
from typing import List, Dict, Tuple, Union, Any
import threading
from datetime import datetime

class DPage(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__int_mutex: threading.Lock = threading.Lock()
        self.__float_mutex: threading.Lock = threading.Lock()
        self.__str_mutex: threading.Lock = threading.Lock()
        self.__bool_mutex: threading.Lock = threading.Lock()
        self.__other_mutex: threading.Lock = threading.Lock()
        self.__int32_map: Dict[int, int] = dict()
        self.__int64_map: Dict[int, int] = dict()
        self.__float_map: Dict[int, float] = dict()
        self.__double_map: Dict[int, float] = dict()
        self.__string_map: Dict[int, str] = dict()
        self.__bool_map: Dict[int, bool] = dict()
        self.__d_int32_map: Dict[int, Union[int, datetime]] = dict()
        self.__d_int64_map: Dict[int, Union[int, datetime]] = dict()
        self.__d_float_map: Dict[int, Union[float, datetime]] = dict()
        self.__d_double_map: Dict[int, Union[float, datetime]] = dict()
        self.__d_string_map: Dict[int, Union[str, datetime]] = dict()
        self.__d_bool_map: Dict[int, Union[bool, datetime]] = dict()

        self.__save_map: Dict[int, Any] = dict()
        self.__get_map: Dict[int, Any] = dict()
        self.__save_other_map: Dict[int, Any] = dict()
        self.__get_other_map: Dict[int, Any] = dict()
        self.__bucket_size = 10000
        self.__update_cb: Any = None
        self.__get_parent_dcache_fun: Any = None
        self.__page_id: str = None

    def init_component(self
                       , logger: logging.Logger
                       , update_cb: Any
                       , get_parent_dcache_fun: Any
                       , page_id: str
                       , save_other_map: Dict
                       , get_other_map: Dict) -> None:
        self.__logger = logger
        self.__get_parent_dcache_fun = get_parent_dcache_fun
        self.__update_cb = update_cb
        self.__save_other_map = save_other_map
        self.__get_other_map = get_other_map
        self.__page_id = page_id

        self.__save_map = {
            0: self.__save_fid_int32
            , 1: self.__save_fid_int64
            , 2: self.__save_fid_float
            , 3: self.__save_fid_double
            , 4: self.__save_fid_string
            , 5: self.__save_fid_other
            , 6: self.__save_fid_bool
            , 7: self.__save_fid_d_int32
            , 8: self.__save_fid_d_int64
            , 9: self.__save_fid_d_float
            , 10: self.__save_fid_d_double
            , 11: self.__save_fid_d_string
            , 12: self.__save_fid_d_bool
        }
        self.__get_map = {
            0: self.__get_fid_int32
            , 1: self.__get_fid_int64
            , 2: self.__get_fid_float
            , 3: self.__get_fid_double
            , 4: self.__get_fid_string
            , 5: self.__get_fid_other
            , 6: self.__get_fid_bool
            , 7: self.__get_fid_d_int32
            , 8: self.__get_fid_d_int64
            , 9: self.__get_fid_d_float
            , 10: self.__get_fid_d_double
            , 11: self.__get_fid_d_string
            , 12: self.__get_fid_d_bool
        }

    def save_fid(self, fid_num: int, fid_value: Any) -> None:
        bucket_num = int((fid_num - 1) / self.__bucket_size)
        if bucket_num not in self.__save_map:
            return None
        is_new = self.__save_map[bucket_num](fid_num=fid_num, fid_value=fid_value)
        if is_new is not None and is_new and self.__update_cb is not None:
            self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__page_id, fid_num=fid_num, fid_value=fid_value)

    def get_page_id(self) -> str:
        return self.__page_id

    def get_fids(self) -> List[Union[int, Any]]:
        result_list: List[Union[int, Any]] = list()
        with self.__int_mutex:
            for fid_num, v in self.__int32_map.items():
                result_list.append((fid_num, v))
            for fid_num, v in self.__d_int32_map.items():
                result_list.append((fid_num, v))

            for fid_num, v in self.__int64_map.items():
                result_list.append((fid_num, v))
            for fid_num, v in self.__d_int64_map.items():
                result_list.append((fid_num, v))

        with self.__float_mutex:
            for fid_num, v in self.__float_map.items():
                result_list.append((fid_num, v))
            for fid_num, v in self.__d_float_map.items():
                result_list.append((fid_num, v))

            for fid_num, v in self.__double_map.items():
                result_list.append((fid_num, v))
            for fid_num, v in self.__d_double_map.items():
                result_list.append((fid_num, v))

        with self.__str_mutex:
            for fid_num, v in self.__string_map.items():
                result_list.append((fid_num, v))
            for fid_num, v in self.__d_string_map.items():
                result_list.append((fid_num, v))

        with self.__bool_mutex:
            for fid_num, v in self.__bool_map.items():
                result_list.append((fid_num, v))
            for fid_num, v in self.__d_bool_map.items():
                result_list.append((fid_num, v))

        return result_list

    def get_fid(self, fid_num: int) -> Union[bool, Any]:
        bucket_num = int((fid_num - 1) / self.__bucket_size)
        if bucket_num not in self.__get_map:
            return (False, None)

        return (True, self.__get_map[bucket_num](fid_num=fid_num))

    def __save_fid_int32(self, fid_num: int, fid_value: int) -> bool:
        return self.__save_fid_imp(mutex=self.__int_mutex, the_map=self.__int32_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_int64(self, fid_num: int, fid_value: int) -> bool:
        return self.__save_fid_imp(mutex=self.__int_mutex, the_map=self.__int64_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_float(self, fid_num: int, fid_value: float) -> bool:
        return self.__save_fid_imp(mutex=self.__float_mutex, the_map=self.__float_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_double(self, fid_num: int, fid_value: float) -> bool:
        return self.__save_fid_imp(mutex=self.__float_mutex, the_map=self.__double_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_string(self, fid_num: int, fid_value: str) -> bool:
        return self.__save_fid_imp(mutex=self.__str_mutex, the_map=self.__str_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_bool(self, fid_num: int, fid_value: float) -> bool:
        return self.__save_fid_imp(mutex=self.__bool_mutex, the_map=self.__bool_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_imp(self, mutex: threading.Lock, the_map: Dict[int, Any], fid_num: int, fid_value: Any) -> bool:
        is_new = False
        with mutex:
            if fid_num not in the_map:
                is_new = True

                the_map[fid_num] = fid_value
            else:
                if the_map[fid_num] == fid_value:
                    is_new = False
                else:
                    is_new = True

                    the_map[fid_num] = fid_value

        return is_new

    def __save_fid_d_int32(self, fid_num: int, fid_value: int) -> bool:
        return self.__save_fid_d_imp(mutex=self.__int_mutex, the_map=self.__d_int32_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_d_int64(self, fid_num: int, fid_value: int) -> bool:
        return self.__save_fid_d_imp(mutex=self.__int_mutex, the_map=self.__d_int64_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_d_float(self, fid_num: int, fid_value: float) -> bool:
        return self.__save_fid_d_imp(mutex=self.__float_mutex, the_map=self.__d_float_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_d_double(self, fid_num: int, fid_value: float) -> bool:
        return self.__save_fid_d_imp(mutex=self.__float_mutex, the_map=self.__d_double_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_d_string(self, fid_num: int, fid_value: str) -> bool:
        return self.__save_fid_d_imp(mutex=self.__str_mutex, the_map=self.__d_str_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_d_bool(self, fid_num: int, fid_value: float) -> bool:
        return self.__save_fid_d_imp(mutex=self.__bool_mutex, the_map=self.__d_bool_map, fid_num=fid_num, fid_value=fid_value)

    def __save_fid_d_imp(self, mutex: threading.Lock, the_map: Dict[int, Any], fid_num: int, fid_value: Any) -> bool:
        is_new = False
        with mutex:
            if fid_num not in the_map:
                is_new = True

                the_map[fid_num] = fid_value
            else:
                if the_map[fid_num][0] == fid_value[0]\
                   or the_map[fid_num][1] == fid_value[1]:
                    is_new = False
                else:
                    is_new = True

                    the_map[fid_num] = fid_value

        return is_new

    def __save_fid_other(self, fid_num: int, fid_value: Any) -> None:
        with self.__other_mutex:
            if fid_num not in self.__save_other_map:
                return None

            self.__save_other_map[fid_num](fid_value)
            return None

    def __get_fid_int32(self, fid_num: int) -> Union[bool, int]:
        return self.__get_fid_imp(mutex=self.__int_mutex, the_map=self.__int32_map, fid_num=fid_num)

    def __get_fid_int64(self, fid_num: int) -> Union[bool, int]:
        return self.__get_fid_imp(mutex=self.__int_mutex, the_map=self.__int64_map, fid_num=fid_num)

    def __get_fid_float(self, fid_num: int) -> Union[bool, float]:
        return self.__get_fid_imp(mutex=self.__float_mutex, the_map=self.__float_map, fid_num=fid_num)

    def __get_fid_double(self, fid_num: int) -> Union[bool, float]:
        return self.__get_fid_imp(mutex=self.__float_mutex, the_map=self.__double_map, fid_num=fid_num)

    def __get_fid_string(self, fid_num: int) -> Union[bool, str]:
        return self.__get_fid_imp(mutex=self.__str_mutex, the_map=self.__str_map, fid_num=fid_num)

    def __get_fid_bool(self, fid_num: int) -> Union[bool, bool]:
        return self.__get_fid_imp(mutex=self.__bool_mutex, the_map=self.__bool_map, fid_num=fid_num)

    def __get_fid_imp(self, mutex: threading.Lock, the_map: Dict[int, Any], fid_num: int) -> Union[bool, Any]:
        with mutex:
            if fid_num in the_map:
                return (True, the_map[fid_num])
            else:
                return (False, None)

    def __get_fid_d_int32(self, fid_num: int) -> Union[bool, int]:
        return self.__get_fid_d_imp(mutex=self.__int_mutex, the_map=self.__d_int32_map, fid_num=fid_num)

    def __get_fid_d_int64(self, fid_num: int) -> Union[bool, int]:
        return self.__get_fid_d_imp(mutex=self.__int_mutex, the_map=self.__d_int64_map, fid_num=fid_num)

    def __get_fid_d_float(self, fid_num: int) -> Union[bool, float]:
        return self.__get_fid_d_imp(mutex=self.__float_mutex, the_map=self.__d_float_map, fid_num=fid_num)

    def __get_fid_d_double(self, fid_num: int) -> Union[bool, float]:
        return self.__get_fid_d_imp(mutex=self.__float_mutex, the_map=self.__d_double_map, fid_num=fid_num)

    def __get_fid_d_string(self, fid_num: int) -> Union[bool, str]:
        return self.__get_fid_d_imp(mutex=self.__str_mutex, the_map=self.__d_str_map, fid_num=fid_num)

    def __get_fid_d_bool(self, fid_num: int) -> Union[bool, bool]:
        return self.__get_fid_d_imp(mutex=self.__bool_mutex, the_map=self.__d_bool_map, fid_num=fid_num)

    def __get_fid_d_imp(self, mutex: threading.Lock, the_map: Dict[int, Any], fid_num: int) -> Union[bool, Union[Any, datetime]]:
        with mutex:
            if fid_num in the_map:
                return (True, the_map[fid_num])
            else:
                return (False, None)

    def __get_fid_other(self, fid_num: int) -> Union[bool, Any]:
        with self.__other_mutex:
            if fid_num in self.__get_other_map:
                return (True, self.__get_other_map[fid_num]())
            else:
                return (False, None)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ts_cop.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ts_cop.proto',
  package='ts_cop',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cts_cop.proto\x12\x06ts_cop\"%\n\x06\x44INT32\x12\r\n\x05value\x18\x01 \x01(\x05\x12\x0c\n\x04time\x18\x02 \x01(\x01\"%\n\x06\x44INT64\x12\r\n\x05value\x18\x01 \x01(\x03\x12\x0c\n\x04time\x18\x02 \x01(\x01\"%\n\x06\x44\x46LOAT\x12\r\n\x05value\x18\x01 \x01(\x02\x12\x0c\n\x04time\x18\x02 \x01(\x01\"&\n\x07\x44\x44OUBLE\x12\r\n\x05value\x18\x01 \x01(\x01\x12\x0c\n\x04time\x18\x02 \x01(\x01\"&\n\x07\x44STRING\x12\r\n\x05value\x18\x01 \x01(\t\x12\x0c\n\x04time\x18\x02 \x01(\x01\"$\n\x05\x44\x42OOL\x12\r\n\x05value\x18\x01 \x01(\x08\x12\x0c\n\x04time\x18\x02 \x01(\x01\"\x13\n\x02\x43I\x12\r\n\x05value\x18\x01 \x01(\t\"%\n\x06\x43IList\x12\x1b\n\x07\x63i_list\x18\x01 \x03(\x0b\x32\n.ts_cop.CI\"\xdb\x01\n\x02SI\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\r\n\x05ratio\x18\x02 \x01(\x01\x12\x11\n\tparent_id\x18\x03 \x01(\t\x12\r\n\x05si_id\x18\x04 \x01(\t\x12\x0f\n\x07\x63reated\x18\x05 \x01(\x01\x12\x13\n\x0blast_update\x18\x06 \x01(\x01\x12\x15\n\rstrategy_name\x18\x07 \x01(\t\x12\x11\n\tlive_mode\x18\x08 \x01(\x08\x12\x13\n\x0bstrategy_id\x18\t \x01(\t\x12\x11\n\tsymbol_id\x18\n \x01(\t\x12\x1c\n\x14parent_strategy_name\x18\x0b \x01(\t\"%\n\x06SIList\x12\x1b\n\x07si_list\x18\x01 \x03(\x0b\x32\n.ts_cop.SI\"\xbe\x03\n\x07TSOrder\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08quantity\x18\x02 \x01(\x01\x12\x15\n\rfill_quantity\x18\x03 \x01(\x01\x12\x11\n\tparent_id\x18\x04 \x01(\t\x12\x10\n\x08order_id\x18\x05 \x01(\t\x12\x19\n\x11platform_order_id\x18\x06 \x01(\t\x12\x0f\n\x07\x63reated\x18\x07 \x01(\x01\x12\x13\n\x0blast_update\x18\x08 \x01(\x01\x12\x15\n\rstrategy_name\x18\t \x01(\t\x12\x11\n\tlive_mode\x18\n \x01(\x08\x12\x13\n\x0bstrategy_id\x18\x0b \x01(\t\x12\x14\n\x0corder_status\x18\x0c \x01(\t\x12\x18\n\x10\x65xecution_broker\x18\r \x01(\t\x12\x17\n\x0f\x63learing_broker\x18\x0e \x01(\t\x12\x0f\n\x07\x61\x63\x63ount\x18\x0f \x01(\t\x12\r\n\x05price\x18\x10 \x01(\x01\x12\x0b\n\x03\x66\x65\x65\x18\x11 \x01(\x01\x12\x10\n\x08\x65xchange\x18\x12 \x01(\t\x12\x0b\n\x03\x63\x63y\x18\x13 \x01(\t\x12\x0f\n\x07\x66\x65\x65_ccy\x18\x14 \x01(\t\x12\x11\n\tsymbol_id\x18\x15 \x01(\t\x12\x1c\n\x14parent_strategy_name\x18\x16 \x01(\t\"2\n\x0bTSOrderList\x12#\n\norder_list\x18\x01 \x03(\x0b\x32\x0f.ts_cop.TSOrder\"\xb7\x03\n\x07TSTrade\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08quantity\x18\x02 \x01(\x01\x12\x11\n\tparent_id\x18\x03 \x01(\t\x12\x10\n\x08trade_id\x18\x04 \x01(\t\x12\x0f\n\x07\x63reated\x18\x05 \x01(\x01\x12\x13\n\x0blast_update\x18\x06 \x01(\x01\x12\x15\n\rstrategy_name\x18\x07 \x01(\t\x12\x11\n\tlive_mode\x18\x08 \x01(\x08\x12\x13\n\x0bstrategy_id\x18\t \x01(\t\x12\x14\n\x0ctrade_status\x18\n \x01(\t\x12\x18\n\x10\x65xecution_broker\x18\x0b \x01(\t\x12\x17\n\x0f\x63learing_broker\x18\x0c \x01(\t\x12\x17\n\x0fsettlement_date\x18\r \x01(\x01\x12\x10\n\x08\x66x_ratio\x18\x0e \x01(\x01\x12\x0f\n\x07\x61\x63\x63ount\x18\x0f \x01(\t\x12\r\n\x05price\x18\x10 \x01(\x01\x12\x0b\n\x03\x66\x65\x65\x18\x11 \x01(\x01\x12\x10\n\x08\x65xchange\x18\x12 \x01(\t\x12\x0b\n\x03\x63\x63y\x18\x13 \x01(\t\x12\x0f\n\x07\x66\x65\x65_ccy\x18\x14 \x01(\t\x12\x11\n\tsymbol_id\x18\x15 \x01(\t\x12\x1c\n\x14parent_strategy_name\x18\x16 \x01(\t\"2\n\x0bTSTradeList\x12#\n\ntrade_list\x18\x01 \x03(\x0b\x32\x0f.ts_cop.TSTrade\"\xfb\x03\n\x06TSDATA\x12\x12\n\nint32_data\x18\x01 \x01(\x05\x12\x12\n\nint64_data\x18\x02 \x01(\x03\x12\x12\n\nfloat_data\x18\x03 \x01(\x02\x12\x13\n\x0b\x64ouble_data\x18\x04 \x01(\x01\x12\x13\n\x0bstring_data\x18\x05 \x01(\t\x12\x11\n\tbool_data\x18\x06 \x01(\x08\x12\'\n\norder_data\x18\x07 \x01(\x0b\x32\x13.ts_cop.TSOrderList\x12\'\n\ntrade_data\x18\x08 \x01(\x0b\x32\x13.ts_cop.TSTradeList\x12\x1f\n\x07si_data\x18\t \x01(\x0b\x32\x0e.ts_cop.SIList\x12\x1f\n\x07\x63i_data\x18\n \x01(\x0b\x32\x0e.ts_cop.CIList\x12$\n\x0c\x64_int32_data\x18\x0b \x01(\x0b\x32\x0e.ts_cop.DINT32\x12$\n\x0c\x64_int64_data\x18\x0c \x01(\x0b\x32\x0e.ts_cop.DINT64\x12$\n\x0c\x64_float_data\x18\r \x01(\x0b\x32\x0e.ts_cop.DFLOAT\x12&\n\rd_double_data\x18\x0e \x01(\x0b\x32\x0f.ts_cop.DDOUBLE\x12&\n\rd_string_data\x18\x0f \x01(\x0b\x32\x0f.ts_cop.DSTRING\x12\"\n\x0b\x64_bool_data\x18\x10 \x01(\x0b\x32\r.ts_cop.DBOOL\"\x90\x03\n\x03\x43op\x12\x0e\n\x06msg_id\x18\x01 \x01(\t\x12%\n\x08msg_type\x18\x02 \x01(\x0e\x32\x13.ts_cop.Cop.MsgType\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x13\n\x0binstance_id\x18\x04 \x01(\x05\x12\x0f\n\x07seq_num\x18\x05 \x01(\x03\x12\x14\n\x0c\x63reated_time\x18\x06 \x01(\x01\x12\x13\n\x0bstrategy_id\x18\x07 \x01(\t\x12*\n\x08\x64\x61ta_map\x18\x08 \x03(\x0b\x32\x18.ts_cop.Cop.DataMapEntry\x1a>\n\x0c\x44\x61taMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\x1d\n\x05value\x18\x02 \x01(\x0b\x32\x0e.ts_cop.TSDATA:\x02\x38\x01\"0\n\x07MsgType\x12\x0b\n\x07Listing\x10\x00\x12\x0c\n\x08Strategy\x10\x01\x12\n\n\x06System\x10\x02\"S\n\x06\x46idNum\x12\t\n\x05\x44ummy\x10\x00\x12\x08\n\x02\x43I\x10\xd1\x86\x03\x12\x08\n\x02SI\x10\xd2\x86\x03\x12\x0b\n\x05Order\x10\xd3\x86\x03\x12\x0b\n\x05Trade\x10\xd4\x86\x03\x12\x10\n\nOrder_Snap\x10\xd5\x86\x03\x62\x06proto3'
)



_COP_MSGTYPE = _descriptor.EnumDescriptor(
  name='MsgType',
  full_name='ts_cop.Cop.MsgType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Listing', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Strategy', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='System', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2353,
  serialized_end=2401,
)
_sym_db.RegisterEnumDescriptor(_COP_MSGTYPE)

_COP_FIDNUM = _descriptor.EnumDescriptor(
  name='FidNum',
  full_name='ts_cop.Cop.FidNum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Dummy', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CI', index=1, number=50001,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SI', index=2, number=50002,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Order', index=3, number=50003,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Trade', index=4, number=50004,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Order_Snap', index=5, number=50005,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2403,
  serialized_end=2486,
)
_sym_db.RegisterEnumDescriptor(_COP_FIDNUM)


_DINT32 = _descriptor.Descriptor(
  name='DINT32',
  full_name='ts_cop.DINT32',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.DINT32.value', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='ts_cop.DINT32.time', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=61,
)


_DINT64 = _descriptor.Descriptor(
  name='DINT64',
  full_name='ts_cop.DINT64',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.DINT64.value', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='ts_cop.DINT64.time', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=63,
  serialized_end=100,
)


_DFLOAT = _descriptor.Descriptor(
  name='DFLOAT',
  full_name='ts_cop.DFLOAT',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.DFLOAT.value', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='ts_cop.DFLOAT.time', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=102,
  serialized_end=139,
)


_DDOUBLE = _descriptor.Descriptor(
  name='DDOUBLE',
  full_name='ts_cop.DDOUBLE',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.DDOUBLE.value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='ts_cop.DDOUBLE.time', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=141,
  serialized_end=179,
)


_DSTRING = _descriptor.Descriptor(
  name='DSTRING',
  full_name='ts_cop.DSTRING',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.DSTRING.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='ts_cop.DSTRING.time', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=181,
  serialized_end=219,
)


_DBOOL = _descriptor.Descriptor(
  name='DBOOL',
  full_name='ts_cop.DBOOL',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.DBOOL.value', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='ts_cop.DBOOL.time', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=221,
  serialized_end=257,
)


_CI = _descriptor.Descriptor(
  name='CI',
  full_name='ts_cop.CI',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.CI.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=259,
  serialized_end=278,
)


_CILIST = _descriptor.Descriptor(
  name='CIList',
  full_name='ts_cop.CIList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ci_list', full_name='ts_cop.CIList.ci_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=280,
  serialized_end=317,
)


_SI = _descriptor.Descriptor(
  name='SI',
  full_name='ts_cop.SI',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='symbol', full_name='ts_cop.SI.symbol', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ratio', full_name='ts_cop.SI.ratio', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_id', full_name='ts_cop.SI.parent_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='si_id', full_name='ts_cop.SI.si_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created', full_name='ts_cop.SI.created', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='ts_cop.SI.last_update', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='ts_cop.SI.strategy_name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='live_mode', full_name='ts_cop.SI.live_mode', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='ts_cop.SI.strategy_id', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol_id', full_name='ts_cop.SI.symbol_id', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_strategy_name', full_name='ts_cop.SI.parent_strategy_name', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=320,
  serialized_end=539,
)


_SILIST = _descriptor.Descriptor(
  name='SIList',
  full_name='ts_cop.SIList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='si_list', full_name='ts_cop.SIList.si_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=541,
  serialized_end=578,
)


_TSORDER = _descriptor.Descriptor(
  name='TSOrder',
  full_name='ts_cop.TSOrder',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='symbol', full_name='ts_cop.TSOrder.symbol', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quantity', full_name='ts_cop.TSOrder.quantity', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fill_quantity', full_name='ts_cop.TSOrder.fill_quantity', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_id', full_name='ts_cop.TSOrder.parent_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_id', full_name='ts_cop.TSOrder.order_id', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='platform_order_id', full_name='ts_cop.TSOrder.platform_order_id', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created', full_name='ts_cop.TSOrder.created', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='ts_cop.TSOrder.last_update', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='ts_cop.TSOrder.strategy_name', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='live_mode', full_name='ts_cop.TSOrder.live_mode', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='ts_cop.TSOrder.strategy_id', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_status', full_name='ts_cop.TSOrder.order_status', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='execution_broker', full_name='ts_cop.TSOrder.execution_broker', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clearing_broker', full_name='ts_cop.TSOrder.clearing_broker', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='account', full_name='ts_cop.TSOrder.account', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='ts_cop.TSOrder.price', index=15,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee', full_name='ts_cop.TSOrder.fee', index=16,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='ts_cop.TSOrder.exchange', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ccy', full_name='ts_cop.TSOrder.ccy', index=18,
      number=19, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee_ccy', full_name='ts_cop.TSOrder.fee_ccy', index=19,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol_id', full_name='ts_cop.TSOrder.symbol_id', index=20,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_strategy_name', full_name='ts_cop.TSOrder.parent_strategy_name', index=21,
      number=22, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=581,
  serialized_end=1027,
)


_TSORDERLIST = _descriptor.Descriptor(
  name='TSOrderList',
  full_name='ts_cop.TSOrderList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='order_list', full_name='ts_cop.TSOrderList.order_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1029,
  serialized_end=1079,
)


_TSTRADE = _descriptor.Descriptor(
  name='TSTrade',
  full_name='ts_cop.TSTrade',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='symbol', full_name='ts_cop.TSTrade.symbol', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quantity', full_name='ts_cop.TSTrade.quantity', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_id', full_name='ts_cop.TSTrade.parent_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trade_id', full_name='ts_cop.TSTrade.trade_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created', full_name='ts_cop.TSTrade.created', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_update', full_name='ts_cop.TSTrade.last_update', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='ts_cop.TSTrade.strategy_name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='live_mode', full_name='ts_cop.TSTrade.live_mode', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='ts_cop.TSTrade.strategy_id', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trade_status', full_name='ts_cop.TSTrade.trade_status', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='execution_broker', full_name='ts_cop.TSTrade.execution_broker', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clearing_broker', full_name='ts_cop.TSTrade.clearing_broker', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='settlement_date', full_name='ts_cop.TSTrade.settlement_date', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fx_ratio', full_name='ts_cop.TSTrade.fx_ratio', index=13,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='account', full_name='ts_cop.TSTrade.account', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='ts_cop.TSTrade.price', index=15,
      number=16, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee', full_name='ts_cop.TSTrade.fee', index=16,
      number=17, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='ts_cop.TSTrade.exchange', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ccy', full_name='ts_cop.TSTrade.ccy', index=18,
      number=19, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee_ccy', full_name='ts_cop.TSTrade.fee_ccy', index=19,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol_id', full_name='ts_cop.TSTrade.symbol_id', index=20,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_strategy_name', full_name='ts_cop.TSTrade.parent_strategy_name', index=21,
      number=22, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1082,
  serialized_end=1521,
)


_TSTRADELIST = _descriptor.Descriptor(
  name='TSTradeList',
  full_name='ts_cop.TSTradeList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trade_list', full_name='ts_cop.TSTradeList.trade_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1523,
  serialized_end=1573,
)


_TSDATA = _descriptor.Descriptor(
  name='TSDATA',
  full_name='ts_cop.TSDATA',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='int32_data', full_name='ts_cop.TSDATA.int32_data', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int64_data', full_name='ts_cop.TSDATA.int64_data', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float_data', full_name='ts_cop.TSDATA.float_data', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double_data', full_name='ts_cop.TSDATA.double_data', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_data', full_name='ts_cop.TSDATA.string_data', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_data', full_name='ts_cop.TSDATA.bool_data', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_data', full_name='ts_cop.TSDATA.order_data', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trade_data', full_name='ts_cop.TSDATA.trade_data', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='si_data', full_name='ts_cop.TSDATA.si_data', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ci_data', full_name='ts_cop.TSDATA.ci_data', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='d_int32_data', full_name='ts_cop.TSDATA.d_int32_data', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='d_int64_data', full_name='ts_cop.TSDATA.d_int64_data', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='d_float_data', full_name='ts_cop.TSDATA.d_float_data', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='d_double_data', full_name='ts_cop.TSDATA.d_double_data', index=13,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='d_string_data', full_name='ts_cop.TSDATA.d_string_data', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='d_bool_data', full_name='ts_cop.TSDATA.d_bool_data', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1576,
  serialized_end=2083,
)


_COP_DATAMAPENTRY = _descriptor.Descriptor(
  name='DataMapEntry',
  full_name='ts_cop.Cop.DataMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ts_cop.Cop.DataMapEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ts_cop.Cop.DataMapEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2289,
  serialized_end=2351,
)

_COP = _descriptor.Descriptor(
  name='Cop',
  full_name='ts_cop.Cop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg_id', full_name='ts_cop.Cop.msg_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg_type', full_name='ts_cop.Cop.msg_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='ts_cop.Cop.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instance_id', full_name='ts_cop.Cop.instance_id', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq_num', full_name='ts_cop.Cop.seq_num', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_time', full_name='ts_cop.Cop.created_time', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='ts_cop.Cop.strategy_id', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_map', full_name='ts_cop.Cop.data_map', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_COP_DATAMAPENTRY, ],
  enum_types=[
    _COP_MSGTYPE,
    _COP_FIDNUM,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2086,
  serialized_end=2486,
)

_CILIST.fields_by_name['ci_list'].message_type = _CI
_SILIST.fields_by_name['si_list'].message_type = _SI
_TSORDERLIST.fields_by_name['order_list'].message_type = _TSORDER
_TSTRADELIST.fields_by_name['trade_list'].message_type = _TSTRADE
_TSDATA.fields_by_name['order_data'].message_type = _TSORDERLIST
_TSDATA.fields_by_name['trade_data'].message_type = _TSTRADELIST
_TSDATA.fields_by_name['si_data'].message_type = _SILIST
_TSDATA.fields_by_name['ci_data'].message_type = _CILIST
_TSDATA.fields_by_name['d_int32_data'].message_type = _DINT32
_TSDATA.fields_by_name['d_int64_data'].message_type = _DINT64
_TSDATA.fields_by_name['d_float_data'].message_type = _DFLOAT
_TSDATA.fields_by_name['d_double_data'].message_type = _DDOUBLE
_TSDATA.fields_by_name['d_string_data'].message_type = _DSTRING
_TSDATA.fields_by_name['d_bool_data'].message_type = _DBOOL
_COP_DATAMAPENTRY.fields_by_name['value'].message_type = _TSDATA
_COP_DATAMAPENTRY.containing_type = _COP
_COP.fields_by_name['msg_type'].enum_type = _COP_MSGTYPE
_COP.fields_by_name['data_map'].message_type = _COP_DATAMAPENTRY
_COP_MSGTYPE.containing_type = _COP
_COP_FIDNUM.containing_type = _COP
DESCRIPTOR.message_types_by_name['DINT32'] = _DINT32
DESCRIPTOR.message_types_by_name['DINT64'] = _DINT64
DESCRIPTOR.message_types_by_name['DFLOAT'] = _DFLOAT
DESCRIPTOR.message_types_by_name['DDOUBLE'] = _DDOUBLE
DESCRIPTOR.message_types_by_name['DSTRING'] = _DSTRING
DESCRIPTOR.message_types_by_name['DBOOL'] = _DBOOL
DESCRIPTOR.message_types_by_name['CI'] = _CI
DESCRIPTOR.message_types_by_name['CIList'] = _CILIST
DESCRIPTOR.message_types_by_name['SI'] = _SI
DESCRIPTOR.message_types_by_name['SIList'] = _SILIST
DESCRIPTOR.message_types_by_name['TSOrder'] = _TSORDER
DESCRIPTOR.message_types_by_name['TSOrderList'] = _TSORDERLIST
DESCRIPTOR.message_types_by_name['TSTrade'] = _TSTRADE
DESCRIPTOR.message_types_by_name['TSTradeList'] = _TSTRADELIST
DESCRIPTOR.message_types_by_name['TSDATA'] = _TSDATA
DESCRIPTOR.message_types_by_name['Cop'] = _COP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DINT32 = _reflection.GeneratedProtocolMessageType('DINT32', (_message.Message,), {
  'DESCRIPTOR' : _DINT32,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.DINT32)
  })
_sym_db.RegisterMessage(DINT32)

DINT64 = _reflection.GeneratedProtocolMessageType('DINT64', (_message.Message,), {
  'DESCRIPTOR' : _DINT64,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.DINT64)
  })
_sym_db.RegisterMessage(DINT64)

DFLOAT = _reflection.GeneratedProtocolMessageType('DFLOAT', (_message.Message,), {
  'DESCRIPTOR' : _DFLOAT,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.DFLOAT)
  })
_sym_db.RegisterMessage(DFLOAT)

DDOUBLE = _reflection.GeneratedProtocolMessageType('DDOUBLE', (_message.Message,), {
  'DESCRIPTOR' : _DDOUBLE,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.DDOUBLE)
  })
_sym_db.RegisterMessage(DDOUBLE)

DSTRING = _reflection.GeneratedProtocolMessageType('DSTRING', (_message.Message,), {
  'DESCRIPTOR' : _DSTRING,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.DSTRING)
  })
_sym_db.RegisterMessage(DSTRING)

DBOOL = _reflection.GeneratedProtocolMessageType('DBOOL', (_message.Message,), {
  'DESCRIPTOR' : _DBOOL,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.DBOOL)
  })
_sym_db.RegisterMessage(DBOOL)

CI = _reflection.GeneratedProtocolMessageType('CI', (_message.Message,), {
  'DESCRIPTOR' : _CI,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.CI)
  })
_sym_db.RegisterMessage(CI)

CIList = _reflection.GeneratedProtocolMessageType('CIList', (_message.Message,), {
  'DESCRIPTOR' : _CILIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.CIList)
  })
_sym_db.RegisterMessage(CIList)

SI = _reflection.GeneratedProtocolMessageType('SI', (_message.Message,), {
  'DESCRIPTOR' : _SI,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.SI)
  })
_sym_db.RegisterMessage(SI)

SIList = _reflection.GeneratedProtocolMessageType('SIList', (_message.Message,), {
  'DESCRIPTOR' : _SILIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.SIList)
  })
_sym_db.RegisterMessage(SIList)

TSOrder = _reflection.GeneratedProtocolMessageType('TSOrder', (_message.Message,), {
  'DESCRIPTOR' : _TSORDER,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSOrder)
  })
_sym_db.RegisterMessage(TSOrder)

TSOrderList = _reflection.GeneratedProtocolMessageType('TSOrderList', (_message.Message,), {
  'DESCRIPTOR' : _TSORDERLIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSOrderList)
  })
_sym_db.RegisterMessage(TSOrderList)

TSTrade = _reflection.GeneratedProtocolMessageType('TSTrade', (_message.Message,), {
  'DESCRIPTOR' : _TSTRADE,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSTrade)
  })
_sym_db.RegisterMessage(TSTrade)

TSTradeList = _reflection.GeneratedProtocolMessageType('TSTradeList', (_message.Message,), {
  'DESCRIPTOR' : _TSTRADELIST,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSTradeList)
  })
_sym_db.RegisterMessage(TSTradeList)

TSDATA = _reflection.GeneratedProtocolMessageType('TSDATA', (_message.Message,), {
  'DESCRIPTOR' : _TSDATA,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.TSDATA)
  })
_sym_db.RegisterMessage(TSDATA)

Cop = _reflection.GeneratedProtocolMessageType('Cop', (_message.Message,), {

  'DataMapEntry' : _reflection.GeneratedProtocolMessageType('DataMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _COP_DATAMAPENTRY,
    '__module__' : 'ts_cop_pb2'
    # @@protoc_insertion_point(class_scope:ts_cop.Cop.DataMapEntry)
    })
  ,
  'DESCRIPTOR' : _COP,
  '__module__' : 'ts_cop_pb2'
  # @@protoc_insertion_point(class_scope:ts_cop.Cop)
  })
_sym_db.RegisterMessage(Cop)
_sym_db.RegisterMessage(Cop.DataMapEntry)


_COP_DATAMAPENTRY._options = None
# @@protoc_insertion_point(module_scope)



import copy
import datetime

def to_timestamp_safe(value) -> float:
    if isinstance(value, datetime.datetime):
        return value.timestamp()
    else:
        return value

class StrategyInsight(object):
    def __init__(self, symbol: str, ratio: float):
        self.symbol: str = symbol
        self.symbol_id: str = ""
        self.ratio: float = ratio
        self.parent_id: str = ""
        self.si_id: str = ""
        self.created: datetime = None
        self.last_update: datetime = None
        self.strategy_name: str = ""
        self.parent_strategy_name: str = ""
        self.live_mode: bool = False
        self.strategy_id: str = ""

    def __eq__(self, other) -> bool:
        if isinstance(other, StrategyInsight):
            return self.symbol == other.symbol\
                and self.symbol_id == other.symbol_id\
                and self.ratio == other.ratio\
                and self.parent_id == other.parent_id\
                and self.si_id == other.si_id\
                and self.created == other.created\
                and self.last_update == other.last_update\
                and self.strategy_name == other.strategy_name\
                and self.parent_strategy_name == other.parent_strategy_name\
                and self.live_mode == other.live_mode\
                and self.strategy_id == other.strategy_id
        else:
            return False

    def __str__(self):
        return f"{self.symbol},{self.ratio}"

    def __deepcopy__(self, memo):
        cls = self.__class__
        result = cls.__new__(cls)
        memo[id(self)] = result
        special_fields = set(["created", "last_update"])
        for k, v in self.__dict__.items():
            if k in special_fields:
                setattr(result, k, datetime.datetime.fromtimestamp(to_timestamp_safe(v)))
            else:
                setattr(result, k, copy.deepcopy(v, memo))

        return result

class TS_Order(object):
    def __init__(self):
        self.symbol: str = ""
        self.symbol_id: str = ""
        self.quantity: float = 0
        self.fill_quantity: float = 0
        self.parent_id: str = ""
        self.order_id: str = ""
        self.platform_order_id: int = 0
        self.created: datetime = None
        self.last_update: datetime = None
        self.strategy_name: str = ""
        self.parent_strategy_name: str = ""
        self.live_mode: bool = False
        self.strategy_id: str = ""
        # https://github.com/QuantConnect/Lean/blob/master/Common/Orders/OrderTypes.cs#L107
        self.order_status: str = ""
        self.execution_broker: str = ""
        self.clearing_broker: str = ""
        self.account: str = ""
        self.price: float = 0
        self.fee: float = 0
        self.fee_ccy: str = ""
        self.exchange: str = ""
        self.ccy: str = ""

    def __eq__(self, other) -> bool:
        if isinstance(other, TS_Order):
            return self.symbol == other.symbol\
                and self.symbol_id == other.symbol_id\
                and self.quantity == other.quantity\
                and self.fill_quantity == other.fill_quantity\
                and self.parent_id == other.parent_id\
                and self.order_id == other.order_id\
                and self.platform_order_id == other.platform_order_id\
                and self.created == other.created\
                and self.last_update == other.last_update\
                and self.strategy_name == other.strategy_name\
                and self.parent_strategy_name == other.parent_strategy_name\
                and self.live_mode == other.live_mode\
                and self.strategy_id == other.strategy_id\
                and self.order_status == other.order_status\
                and self.execution_broker == other.execution_broker\
                and self.clearing_broker == other.clearing_broker\
                and self.account == other.account\
                and self.price == other.price\
                and self.fee == other.fee\
                and self.fee_ccy == other.fee_ccy\
                and self.exchange == other.exchange\
                and self.ccy == other.ccy
        else:
            return False

    def __str__(self):
        return f"{self.symbol},{self.quantity},{self.fill_quantity}"

    def __deepcopy__(self, memo):
        cls = self.__class__
        result = cls.__new__(cls)
        memo[id(self)] = result
        special_fields = set(["created", "last_update"])
        for k, v in self.__dict__.items():
            if k in special_fields:
                setattr(result, k, datetime.datetime.fromtimestamp(to_timestamp_safe(v)))
            else:
                setattr(result, k, copy.deepcopy(v, memo))

        return result

class TS_Trade(object):
    def __init__(self):
        self.symbol: str = ""
        self.symbol_id: str = ""
        self.quantity: float = 0
        self.parent_id: str = ""
        self.trade_id: str = ""
        self.created: datetime = None
        self.last_update: datetime = None
        self.strategy_name: str = ""
        self.parent_strategy_name: str = ""
        self.live_mode: bool = False
        self.strategy_id: str = ""
        # https://github.com/QuantConnect/Lean/blob/master/Common/Orders/OrderTypes.cs#L107
        self.trade_status: str = ""
        self.execution_broker: str = ""
        self.clearing_broker: str = ""
        self.settlement_date: datetime = None
        self.fx_ratio: float = 0
        self.account: str = ""
        self.price: float = 0
        self.fee: float = 0
        self.fee_ccy: str = ""
        self.exchange: str = ""
        self.ccy: str = ""

    def __eq__(self, other) -> bool:
        if isinstance(other, TS_Trade):
            return self.symbol == other.symbol\
                and self.symbol_id == other.symbol_id\
                and self.quantity == other.quantity\
                and self.parent_id == other.parent_id\
                and self.trade_id == other.trade_id\
                and self.created == other.created\
                and self.last_update == other.last_update\
                and self.strategy_name == other.strategy_name\
                and self.parent_strategy_name == other.parent_strategy_name\
                and self.live_mode == other.live_mode\
                and self.strategy_id == other.strategy_id\
                and self.trade_status == other.trade_status\
                and self.execution_broker == other.execution_broker\
                and self.clearing_broker == other.clearing_broker\
                and self.settlement_date == other.settlement_date\
                and self.fx_ratio == other.fx_ratio\
                and self.account == other.account\
                and self.price == other.price\
                and self.fee == other.fee\
                and self.fee_ccy == other.fee_ccy\
                and self.exchange == other.exchange\
                and self.ccy == other.ccy
        else:
            return False

    def __str__(self):
        return f"{self.symbol},{self.quantity}"

    def __deepcopy__(self, memo):
        cls = self.__class__
        result = cls.__new__(cls)
        memo[id(self)] = result
        special_fields = set(["created", "last_update", "settlement_date"])
        for k, v in self.__dict__.items():
            if k in special_fields:
                setattr(result, k, datetime.datetime.fromtimestamp(to_timestamp_safe(v)))
            else:
                setattr(result, k, copy.deepcopy(v, memo))

        return result



import logging
import os
import time

import sfdevtools.observability.log_helper as lh
import sfdevtools.data_control.TSDataCtrl as TSDataCtrl
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc


def test_cb(topic: str, cop: ts_cop_pb2.Cop):
    logger: logging.Logger = lh.init_logger(logger_name="TSDataCtrl_sub_logger", is_json_output=False)

    logger.info(f"topic: {topic} cop: {cop}")

def main():
    logger: logging.Logger = lh.init_logger(logger_name="TSDataCtrl_sub_logger", is_json_output=False)
    env_v = {"proxy_host": os.getenv("P_HOST", default="0.0.0.0")
             , "proxy_in_port": os.getenv("P_IN_PORT", default="5557")
             , "proxy_out_port": os.getenv("P_OUT_PORT", default="5558")}
    logger.info(env_v)

    data_ctrl: TSDataCtrl = TSDataCtrl.TSDataCtrl()
    data_ctrl.init_component(logger=logger)
    data_ctrl.init_subscribe(sub_host=env_v["proxy_host"]
                             , sub_port=env_v["proxy_out_port"])
    data_ctrl.reg_sub_topic("RSA_True")
    data_ctrl.reg_callback(cb_fun=test_cb)
    data_ctrl.start_subscription()

    num_count = 0
    while True:
        time.sleep(1)

    data_ctrl.stop_subscription()
    data_ctrl.reg_callback(cb_fun=None)

if __name__ == '__main__':
    main()



import threading
from typing import List, Dict, Tuple
import logging
from functools import partial
import signal

from sfdevtools.devTools.MsgQ import MsgQ

class FuncFifoQ(object):
    def __init__(self
                 , logger: logging.Logger
                 , pool_size: int = 1):
        self.__logger: logging.Logger = logger
        self.__threads: List[threading.Thread] = list()
        self.__msg_q: MsgQ = MsgQ()
        self.__mutex: threading.Lock = threading.Lock()
        self.__is_running: bool = False

        # init threads
        for i in range(pool_size):
            self.__threads.append(threading.Thread(target=self.main))

        signal.signal(signal.SIGINT, self.__cleanup)

    def start_q(self) -> None:
        with self.__mutex:
            self.__is_running = True

        for th in self.__threads:
            th.start()

    def stop_q(self) -> None:
        if not self.__is_q_running():
            return

        with self.__mutex:
            self.__is_running = False

        # notify all the threads
        for th in self.__threads:
            self.push_func(partial(self.__dummy_func))

        for th in self.__threads:
            th.join()

    def push_func(self, user_func):
        self.__msg_q.push(user_func)

    def main(self):
        self.__logger.info("Start")

        while True:
            if not self.__is_q_running():
                break

            user_func = self.__msg_q.get()
            user_func()

            if not self.__is_q_running():
                break

        self.__logger.info("End")

    def __is_q_running(self) -> bool:
        with self.__mutex:
            return self.__is_running

    def __dummy_func(self) -> None:
        pass

    def __cleanup(self, signum, frame):
        self.stop_q()
        exit(0)



from typing import List, Dict, Any
import logging
import threading
import signal

import sfdevtools.data_control.TSDataCtrl as TSDataCtrl
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc
from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade
import sfdevtools.data_cache.DConverter as dconv
from sfdevtools.data_cache.DCacheManager import DCacheManager
from sfdevtools.devTools.AtomicData import AtomicData
from sfdevtools.devTools.DiscontinueSeelp import DiscontinueSeelp

class TSBackgroundCtrl(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__bg_cycle_sec: int = 600
        self.__bg_interval_sec: int = 20
        self.__dcache_m: DCacheManager = None
        self.__is_running: AtomicData = AtomicData()
        self.__is_running.save(user_data=False)
        self.__dsleep: DiscontinueSeelp = None
        self.__send_cop_fun: Any = None

        self.__page_list: List[Union[str, str]] = list()
        self.__strgy_list: List[str] = list()

        self.__bg_th: threading.Thread = threading.Thread(target=self.__main)

        signal.signal(signal.SIGINT, self.__cleanup)

    def init_component(self
                       , logger: logging.Logger
                       , dcache_m: DCacheManager
                       , send_cop_fun: Any
                       , bg_cycle_sec: int = 600
                       , bg_interval_sec: int = 20):
        self.__logger = logger
        self.__dcache_m = dcache_m
        self.__send_cop_fun = send_cop_fun
        self.__bg_cycle_sec = bg_cycle_sec
        self.__bg_interval_sec = bg_interval_sec
        self.__dsleep = DiscontinueSeelp(sleep_time_ms=self.__bg_interval_sec * 1000)

    def start_bg_cycle(self):
        self.__is_running.save(user_data=True)
        self.__bg_th.start()

    def stop_bg_cycle(self):
        self.__is_running.save(user_data=False)
        self.__bg_th.join()

    def __send_bg(self):
        if None is not self.__send_cop_fun:
            return
        # handle page
        if len(self.__page_list) == 0:
            self.__page_list = self.__dcache_m.get_page_name()

        num_page_needed_to_send = max(int(self.__dcache_m.get_total_page_count() / self.__bg_cycle_sec + 0.5), 1)
        for ele in self.__page_list[:num_page_needed_to_send]:
            cache_name = ele[0]
            page_id = ele[1]

            is_page_exist, dpage = self.__dcache_m.get_page(cache_name=cache_name
                                                            , page_id=page_id)
            if not is_page_exist:
                continue

            # convert page to cop
            cop = dconv.conv_page_2_cop(dpage=dpage)
            self.__send_cop_fun(msg_id=page_id, msg_type=ts_cop_pb2.Cop.MsgType.Listing, cop=cop)

        del self.__page_list[:num_page_needed_to_send]

        # hanlde strategy
        if len(self.__strgy_list) == 0:
            self.__strgy_list = self.__dcache_m.get_cache_name()

        num_strgy_needed_to_send = max(int(self.__dcache_m.get_total_cache_count() / self.__bg_cycle_sec + 0.5), 1)
        for cache_name in self.__strgy_list:
            is_strgy_exist, strgy = self.__dcache_m.get_strgy(cache_name=cache_name)

            if not is_strgy_exist:
                continue

            # convert strategy to cop
            cop = dconv.conv_strg_2_cop(strgy=strgy)
            self.__send_cop_fun(msg_id=cache_name, msg_type=ts_cop_pb2.Cop.MsgType.Strategy, cop=cop)

    def __main(self):
        self.__logger.info("Start")
        while self.__is_running.get():
            if not self.__is_running.get():
                break

            self.__dsleep.sleep(user_fun=self.__is_running.get)
            if not self.__is_running.get():
                break

            self.__send_bg()

        self.__logger.info("End")

    def __cleanup(self, signum, frame):
        self.stop_bg_cycle()
        exit(0)



import logging
import threading
from typing import List, Dict, Any, Union

import sfdevtools.data_cache.DTimelyCache as TimelyCache
from sfdevtools.data_cache.DComponents import TS_Order

class DOrderCache(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__mutex: threading.Lock = None
        self.__db_id_hist: TimelyCache.TimelyCache_Hist = None # key: order_id, value: db_record_id
        self.__order_hist: TimelyCache.TimelyCache_Hist = None # key: order_id, value: TS_Order
        self.__order_snapshot: TimelyCache.TimelyCache_Snapshot = None # key: platform_order_id, value: TS_Order
        self.__eq_fun: Any = None

    def init_component(self
                       , logger: logging.Logger
                       , order_hist: TimelyCache.TimelyCache_Hist
                       , order_snapshot: TimelyCache.TimelyCache_Snapshot
                       , db_id_hist: TimelyCache.TimelyCache_Hist
                       , eq_fun: Any = None) -> None:
        self.__logger = logger
        self.__order_hist = order_hist
        self.__order_snapshot = order_snapshot
        self.__db_id_hist = db_id_hist
        self.__mutex = threading.Lock()
        self.__eq_fun = eq_fun

    def save_orders(self, orders=List[TS_Order]) -> List[bool]:
        is_new_list: List[bool] = list()
        if len(orders) == 0:
            return [False]

        with self.__mutex:
            is_need_full_update = False
            for order in orders:
                is_new_list.append(self.__save_order_imp(order=order))
                if is_new_list[-1]:
                    is_need_full_update = True

            if is_need_full_update:
                for order in orders:
                    self.__order_snapshot.upsert_ele(key=order.platform_order_id, value=order)

            return is_new_list
        return [False] * len(orders)

    def save_order(self, order: TS_Order) -> bool:
        with self.__mutex:
            is_new = self.__save_order_imp(order=order)
            if is_new:
                self.__order_snapshot.upsert_ele(key=order.platform_order_id, value=order)

            return is_new
        return False

    def save_db_record_id(self, order: TS_Order, db_record_id: str) -> bool:
        is_new = False

        with self.__mutex:
            if not self.__db_id_hist.is_exist(key=order.order_id):
                # found new record
                is_new = True

                self.__db_id_hist.upsert_ele(key=order.order_id, value=db_record_id)
            else:
                if self.__db_id_hist.get_ele(key=order.order_id) == db_record_id:
                    # found old record
                    is_new = False
                else:
                    # found new record
                    is_new = True

                    self.__db_id_hist.upsert_ele(key=order.order_id, value=db_record_id)

        return is_new

    def is_order_exist(self, order: TS_Order) -> bool:
        with self.__mutex:
            return self.__order_hist.is_exist(key=order.order_id)

    def is_db_record_id_exist(self, order: TS_Order) -> bool:
        with self.__mutex:
            if self.__db_id_hist.is_exist(key=order.platform_order_id)\
               and self.__db_id_hist.get_ele(key=order.platform_order_id) is not None:
                return True
            else:
                return False

    def get_order(self, order_id: int) -> TS_Order:
        with self.__mutex:
            if self.__order_hist.is_exist(key=order_id):
                return self.__order_hist.get_ele(key=order_id)
            else:
                return None

    def get_orders(self, last_n: int = 10) -> List[TS_Order]:
        with self.__mutex:
            return self.__order_hist.get_last_n_records_in_time_series(n=last_n)

    def get_order_latest_snapshot(self, platform_order_id: str) -> TS_Order:
        with self.__mutex:
            if self.__order_snapshot.is_exist(key=platform_order_id):
                return self.__order_snapshot.get_ele(key=platform_order_id)
            else:
                return None

    def get_db_record_id(self, order_id: str) -> str:
        with self.__mutex:
            if self.__db_id_hist.is_exist(key=order_id):
                return self.__db_id_hist.get_ele(key=order_id)
            else:
                return None

    def __save_order_imp(self, order: TS_Order) -> bool:
        is_new = False
        if not self.__order_hist.is_exist(key=order.order_id):
            # found new record
            is_new = True

            self.__order_hist.upsert_ele(key=order.order_id, value=order)
            self.__db_id_hist.upsert_ele(key=order.order_id, value=None)
        else:
            if (self.__eq_fun is not None and self.__eq_fun(first=self.__order_hist.get_ele(key=order.order_id), second=order)\
                or self.__order_hist.get_ele(key=order.order_id) == order):
                # found old record
                is_new = False
            else:
                # found new record
                is_new = True

                self.__order_hist.upsert_ele(key=order.order_id, value=order)
                self.__db_id_hist.upsert_ele(key=order.order_id, value=None)

        return is_new



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: eagle.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='eagle.proto',
  package='eagle',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0b\x65\x61gle.proto\x12\x05\x65\x61gle\"c\n\x19GetBacktestWarmupData_Msg\x12\x13\n\x0bstrategy_id\x18\x01 \x01(\t\x12\x0e\n\x06msg_id\x18\x02 \x01(\t\x12\x10\n\x08msg_type\x18\x03 \x01(\x05\x12\x0f\n\x07\x66id_num\x18\x04 \x01(\x05\"+\n\x1bGetBacktestWarmupData_Reply\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"*\n\x10GetUSLEIData_Msg\x12\x16\n\x0elookback_month\x18\x01 \x01(\x05\"\x8f\x01\n\x12GetUSLEIData_Reply\x12\x44\n\x0fus_lei_data_map\x18\x01 \x03(\x0b\x32+.eagle.GetUSLEIData_Reply.UsLeiDataMapEntry\x1a\x33\n\x11UsLeiDataMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\".\n\x17GetDataByStrategyId_Msg\x12\x13\n\x0bstrategy_id\x18\x01 \x01(\t\"\x80\x01\n\x19GetDataByStrategyId_Reply\x12\x0b\n\x03\x63\x66g\x18\x01 \x03(\t\x12\n\n\x02\x63i\x18\x02 \x03(\t\x12\n\n\x02si\x18\x03 \x01(\t\x12\x10\n\x08ord_snap\x18\x04 \x01(\t\x12\x10\n\x08ord_hist\x18\x05 \x01(\t\x12\x0b\n\x03trd\x18\x06 \x01(\t\x12\r\n\x05price\x18\x07 \x01(\t\"\x17\n\x04Ping\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x17\n\x04Pong\x12\x0f\n\x07message\x18\x01 \x01(\t2\xac\x02\n\x05\x45\x61gle\x12\'\n\x0bHealthCheck\x12\x0b.eagle.Ping\x1a\x0b.eagle.Pong\x12W\n\x13GetDataByStrategyId\x12\x1e.eagle.GetDataByStrategyId_Msg\x1a .eagle.GetDataByStrategyId_Reply\x12]\n\x15GetBacktestWarmupData\x12 .eagle.GetBacktestWarmupData_Msg\x1a\".eagle.GetBacktestWarmupData_Reply\x12\x42\n\x0cGetUSLEIData\x12\x17.eagle.GetUSLEIData_Msg\x1a\x19.eagle.GetUSLEIData_Replyb\x06proto3'
)




_GETBACKTESTWARMUPDATA_MSG = _descriptor.Descriptor(
  name='GetBacktestWarmupData_Msg',
  full_name='eagle.GetBacktestWarmupData_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='eagle.GetBacktestWarmupData_Msg.strategy_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg_id', full_name='eagle.GetBacktestWarmupData_Msg.msg_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg_type', full_name='eagle.GetBacktestWarmupData_Msg.msg_type', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fid_num', full_name='eagle.GetBacktestWarmupData_Msg.fid_num', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22,
  serialized_end=121,
)


_GETBACKTESTWARMUPDATA_REPLY = _descriptor.Descriptor(
  name='GetBacktestWarmupData_Reply',
  full_name='eagle.GetBacktestWarmupData_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='eagle.GetBacktestWarmupData_Reply.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=123,
  serialized_end=166,
)


_GETUSLEIDATA_MSG = _descriptor.Descriptor(
  name='GetUSLEIData_Msg',
  full_name='eagle.GetUSLEIData_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lookback_month', full_name='eagle.GetUSLEIData_Msg.lookback_month', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=168,
  serialized_end=210,
)


_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY = _descriptor.Descriptor(
  name='UsLeiDataMapEntry',
  full_name='eagle.GetUSLEIData_Reply.UsLeiDataMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='eagle.GetUSLEIData_Reply.UsLeiDataMapEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='eagle.GetUSLEIData_Reply.UsLeiDataMapEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=305,
  serialized_end=356,
)

_GETUSLEIDATA_REPLY = _descriptor.Descriptor(
  name='GetUSLEIData_Reply',
  full_name='eagle.GetUSLEIData_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='us_lei_data_map', full_name='eagle.GetUSLEIData_Reply.us_lei_data_map', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=213,
  serialized_end=356,
)


_GETDATABYSTRATEGYID_MSG = _descriptor.Descriptor(
  name='GetDataByStrategyId_Msg',
  full_name='eagle.GetDataByStrategyId_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='eagle.GetDataByStrategyId_Msg.strategy_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=358,
  serialized_end=404,
)


_GETDATABYSTRATEGYID_REPLY = _descriptor.Descriptor(
  name='GetDataByStrategyId_Reply',
  full_name='eagle.GetDataByStrategyId_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cfg', full_name='eagle.GetDataByStrategyId_Reply.cfg', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ci', full_name='eagle.GetDataByStrategyId_Reply.ci', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='si', full_name='eagle.GetDataByStrategyId_Reply.si', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ord_snap', full_name='eagle.GetDataByStrategyId_Reply.ord_snap', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ord_hist', full_name='eagle.GetDataByStrategyId_Reply.ord_hist', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trd', full_name='eagle.GetDataByStrategyId_Reply.trd', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='eagle.GetDataByStrategyId_Reply.price', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=407,
  serialized_end=535,
)


_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='eagle.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='eagle.Ping.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=537,
  serialized_end=560,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='eagle.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='eagle.Pong.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=562,
  serialized_end=585,
)

_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY.containing_type = _GETUSLEIDATA_REPLY
_GETUSLEIDATA_REPLY.fields_by_name['us_lei_data_map'].message_type = _GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY
DESCRIPTOR.message_types_by_name['GetBacktestWarmupData_Msg'] = _GETBACKTESTWARMUPDATA_MSG
DESCRIPTOR.message_types_by_name['GetBacktestWarmupData_Reply'] = _GETBACKTESTWARMUPDATA_REPLY
DESCRIPTOR.message_types_by_name['GetUSLEIData_Msg'] = _GETUSLEIDATA_MSG
DESCRIPTOR.message_types_by_name['GetUSLEIData_Reply'] = _GETUSLEIDATA_REPLY
DESCRIPTOR.message_types_by_name['GetDataByStrategyId_Msg'] = _GETDATABYSTRATEGYID_MSG
DESCRIPTOR.message_types_by_name['GetDataByStrategyId_Reply'] = _GETDATABYSTRATEGYID_REPLY
DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetBacktestWarmupData_Msg = _reflection.GeneratedProtocolMessageType('GetBacktestWarmupData_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETBACKTESTWARMUPDATA_MSG,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.GetBacktestWarmupData_Msg)
  })
_sym_db.RegisterMessage(GetBacktestWarmupData_Msg)

GetBacktestWarmupData_Reply = _reflection.GeneratedProtocolMessageType('GetBacktestWarmupData_Reply', (_message.Message,), {
  'DESCRIPTOR' : _GETBACKTESTWARMUPDATA_REPLY,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.GetBacktestWarmupData_Reply)
  })
_sym_db.RegisterMessage(GetBacktestWarmupData_Reply)

GetUSLEIData_Msg = _reflection.GeneratedProtocolMessageType('GetUSLEIData_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETUSLEIDATA_MSG,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.GetUSLEIData_Msg)
  })
_sym_db.RegisterMessage(GetUSLEIData_Msg)

GetUSLEIData_Reply = _reflection.GeneratedProtocolMessageType('GetUSLEIData_Reply', (_message.Message,), {

  'UsLeiDataMapEntry' : _reflection.GeneratedProtocolMessageType('UsLeiDataMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY,
    '__module__' : 'eagle_pb2'
    # @@protoc_insertion_point(class_scope:eagle.GetUSLEIData_Reply.UsLeiDataMapEntry)
    })
  ,
  'DESCRIPTOR' : _GETUSLEIDATA_REPLY,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.GetUSLEIData_Reply)
  })
_sym_db.RegisterMessage(GetUSLEIData_Reply)
_sym_db.RegisterMessage(GetUSLEIData_Reply.UsLeiDataMapEntry)

GetDataByStrategyId_Msg = _reflection.GeneratedProtocolMessageType('GetDataByStrategyId_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETDATABYSTRATEGYID_MSG,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.GetDataByStrategyId_Msg)
  })
_sym_db.RegisterMessage(GetDataByStrategyId_Msg)

GetDataByStrategyId_Reply = _reflection.GeneratedProtocolMessageType('GetDataByStrategyId_Reply', (_message.Message,), {
  'DESCRIPTOR' : _GETDATABYSTRATEGYID_REPLY,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.GetDataByStrategyId_Reply)
  })
_sym_db.RegisterMessage(GetDataByStrategyId_Reply)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), {
  'DESCRIPTOR' : _PING,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.Ping)
  })
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), {
  'DESCRIPTOR' : _PONG,
  '__module__' : 'eagle_pb2'
  # @@protoc_insertion_point(class_scope:eagle.Pong)
  })
_sym_db.RegisterMessage(Pong)


_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY._options = None

_EAGLE = _descriptor.ServiceDescriptor(
  name='Eagle',
  full_name='eagle.Eagle',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=588,
  serialized_end=888,
  methods=[
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='eagle.Eagle.HealthCheck',
    index=0,
    containing_service=None,
    input_type=_PING,
    output_type=_PONG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDataByStrategyId',
    full_name='eagle.Eagle.GetDataByStrategyId',
    index=1,
    containing_service=None,
    input_type=_GETDATABYSTRATEGYID_MSG,
    output_type=_GETDATABYSTRATEGYID_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBacktestWarmupData',
    full_name='eagle.Eagle.GetBacktestWarmupData',
    index=2,
    containing_service=None,
    input_type=_GETBACKTESTWARMUPDATA_MSG,
    output_type=_GETBACKTESTWARMUPDATA_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUSLEIData',
    full_name='eagle.Eagle.GetUSLEIData',
    index=3,
    containing_service=None,
    input_type=_GETUSLEIDATA_MSG,
    output_type=_GETUSLEIDATA_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_EAGLE)

DESCRIPTOR.services_by_name['Eagle'] = _EAGLE

# @@protoc_insertion_point(module_scope)



import logging
import threading
from typing import List, Dict, Tuple, Union, Any
import copy
import json
import hashlib

from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade
from sfdevtools.data_cache.DPage import DPage
import sfdevtools.data_cache.DTimelyCache as TimelyCache
import sfdevtools.data_cache.DOrderCache as DOrderCache
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc

class DStrategy(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__max_hist_orders: int = 1000
        self.__max_hist_trades: int = 1000
        self.__max_hist_si: int = 1000
        self.__si_cache: TimelyCache.TimelyCache_Hist = TimelyCache.TimelyCache_Hist() # key: si_id --- keep a limited list of latest records
        self.__ci_id: str = ""
        self.__ci_cache: TimelyCache.TimelyCache_Hist = TimelyCache.TimelyCache_Hist() # key: ci_id --- keep a limited list of latest records
        self.__order_cache: DOrderCache.DOrderCache = DOrderCache.DOrderCache()
        self.__trade_cache: TimelyCache.TimelyCache_Hist = TimelyCache.TimelyCache_Hist() # key: trade_id --- keep latest records in hash
        self.__ci_mutex: threading.Lock = threading.Lock()
        self.__si_mutex: threading.Lock = threading.Lock()
        self.__update_cb: Any = None
        self.__get_parent_dcache_fun: Any = None
        self.__strategy_name: str = None

        self.__strategy_page: DPage = DPage()

    def init_component(self
                       , logger: logging.Logger
                       , update_cb: Any
                       , strategy_name: str
                       , get_parent_dcache_fun: Any
                       , max_hist_orders: int = 1000
                       , max_hist_trades: int = 1000
                       , max_hist_si: int = 1000) -> None:
        self.__logger = logger
        self.__strategy_name = strategy_name
        self.__get_parent_dcache_fun = get_parent_dcache_fun
        self.__update_cb = update_cb
        self.__max_hist_orders = max_hist_orders
        self.__max_hist_trades = max_hist_trades
        self.__max_hist_si = max_hist_si
        self.__strategy_page.init_component(logger=self.__logger
                                            , update_cb=self.__update_cb
                                            , page_id=f"page.{self.__strategy_name}"
                                            , get_parent_dcache_fun=self.__get_parent_dcache_fun
                                            , save_other_map={
                                                ts_cop_pb2.Cop.FidNum.CI: self.save_ci
                                                , ts_cop_pb2.Cop.FidNum.SI: self.save_si
                                                , ts_cop_pb2.Cop.FidNum.Order: self.save_orders
                                                , ts_cop_pb2.Cop.FidNum.Trade: self.save_trades
                                            }
                                            , get_other_map={
                                                ts_cop_pb2.Cop.FidNum.CI: self.get_ci
                                                , ts_cop_pb2.Cop.FidNum.SI: self.get_si
                                                , ts_cop_pb2.Cop.FidNum.Order: self.get_orders
                                                , ts_cop_pb2.Cop.FidNum.Trade: self.get_trades
                                            })
        self.__order_cache.init_component(logger=self.__logger
                                              , order_hist=TimelyCache.TimelyCache_Hist()
                                              , order_snapshot=TimelyCache.TimelyCache_Snapshot()
                                              , db_id_hist=TimelyCache.TimelyCache_Hist())

    def save_si(self, si: List[StrategyInsight]) -> bool:
        if len(si) == 0:
            return True
        si_sorted = sorted(si, key=lambda x: x.symbol_id)
        is_new = self.__si_cache.upsert_ele(key=si_sorted[0].si_id, value=si_sorted)
        if is_new and self.__update_cb is not None:
            self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.SI, fid_value=si_sorted)

        return True

    def is_latest_si(self, si: List[StrategyInsight]) -> bool:
        if self.__si_cache.size() < 1:
            return False
        return self.__is_same_si(si1=self.__si_cache.get_records_by_index(-1), si2=si)

    def is_ci_exist(self, ci: Dict[str, Any]) -> bool:
        return self.__ci_cache.is_exist(key=hashlib.md5(json.dumps(ci, sort_keys=True).encode("utf-8")).digest())

    def is_si_exist(self, si: List[StrategyInsight]) -> bool:
        if len(si) == 0:
            return False
        return self.__si_cache.is_exist(key=si[0].si_id)

    def get_si(self) -> List[StrategyInsight]:
        return self.__si_cache.get_records_by_index(-1)

    def get_all_si(self) -> List[List[StrategyInsight]]:
        return self.__si_cache.get_records_in_time_series()

    def save_ci(self, ci: Dict[str, Any]) -> None:
        is_new = self.__ci_cache.upsert_ele(key=hashlib.md5(json.dumps(ci
                                                                       , sort_keys=True).encode("utf-8")).digest()
                                            , value=json.dumps(ci, sort_keys=True))

        if is_new and self.__update_cb is not None:
            self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.CI, fid_value=ci)

    def get_ci(self) -> Dict[str, Any]:
        if self.__ci_cache.size() < 1:
            return None
        return json.loads(self.__ci_cache.get_records_by_index(idx=-1))

    def get_all_ci(self) -> List[Dict[str, Any]]:
        return [json.loads(ci) for ci in self.__ci_cache.get_records_in_time_series()]

    def save_ci_id(self, ci_id: str) -> None:
        with self.__ci_mutex:
            self.__ci_id = ci_id

    def get_ci_id(self) -> str:
        with self.__ci_mutex:
            return self.__ci_id

    def save_trade(self, trade: TS_Trade) -> None:
        is_new = self.__trade_cache.upsert_ele(key=trade.trade_id, value=trade)
        if is_new and self.__update_cb is not None:
            self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.Trade, fid_value=trade)

    def save_trades(self, trades: List[TS_Trade]) -> None:
        is_new_list: List[bool] = self.__trade_cache.upsert_multi(keys=[trade.trade_id for trade in trades], values=trades)
        if self.__update_cb is None:
            return None
        for is_new, trade in zip(is_new_list, trades):
            if is_new:
                self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.Trade, fid_value=trade)

    def is_trade_exist(self, trade: TS_Trade) -> bool:
        return self.__trade_cache.is_exist(key=trade.trade_id)

    def get_trade(self, trade_id: str) -> Union[bool, TS_Trade]:
        trade = self.__trade_cache.get_ele(key=trade_id)
        if None is trade:
            return (False, None)
        else:
            return (True, trade)

    def get_trades(self, last_n: int) -> List[TS_Trade]:
        return self.__trade_cache.get_last_n_records_in_time_series(n=last_n)

    def save_orders(self, orders: List[TS_Order]) -> None:
        is_new_list: List[bool] = self.__order_cache.save_orders(orders=orders)
        if self.__update_cb is None:
            return None
        for is_new, order in zip(is_new_list, orders):
            if is_new:
                self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.Order, fid_value=order)
                order_snap = self.__order_cache.get_order_latest_snapshot(platform_order_id=order.platform_order_id)
                self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.Order_Snap, fid_value=order_snap)

    def save_order(self, order: TS_Order) -> None:
        is_new = self.__order_cache.save_order(order=order)
        if is_new and self.__update_cb is not None:
            self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.Order, fid_value=order)
            order_snap = self.__order_cache.get_order_latest_snapshot(platform_order_id=order.platform_order_id)
            self.__update_cb(dcache=self.__get_parent_dcache_fun(), msg_id=self.__strategy_name, fid_num=ts_cop_pb2.Cop.FidNum.Order_Snap, fid_value=order_snap)

    def save_db_record_id(self, order: TS_Order, db_record_id: str) -> None:
        self.__order_cache.save_db_record_id(order=order, db_record_id=db_record_id)

    def is_order_exist(self, order: TS_Order) -> bool:
        return self.__order_cache.is_order_exist(order=order)

    def is_db_record_id_exist(self, order: TS_Order) -> bool:
        return self.__order_cache.is_db_record_id_exist(order=order)

    def get_strategy_name(self) -> str:
        return self.__strategy_name

    def get_order(self, order_id: str) -> Union[bool, TS_Order]:
        order = self.__order_cache.get_order(order_id=order_id)
        if None is order:
            return (False, None)
        else:
            return (True, order)

    def get_orders(self, last_n: int) -> List[TS_Order]:
        return self.__order_cache.get_orders(last_n=last_n)

    def get_order_latest_snapshot(self, platform_order_id: str) -> Union[bool, TS_Order]:
        order = self.__order_cache.get_order_latest_snapshot(platform_order_id=platform_order_id)
        if None is order:
            return (False, None)
        else:
            return (True, order)

    def get_db_record_id(self, order_id: str) -> Union[bool, str]:
        db_record_id = self.__order_cache.get_db_record_id(order_id=order_id)
        if None is db_record_id:
            return (False, None)
        else:
            return (True, db_record_id)

    def save_fid(self, fid_num: int, fid_value: Any) -> None:
        self.__strategy_page.save_fid(fid_num=fid_num, fid_value=fid_value)

    def get_fids(self) -> List[Union[int, Any]]:
        return self.__strategy_page.get_fids()

    def get_fid(self, fid_num: int) -> Union[bool, Any]:
        return self.__strategy_page.get_fid(fid_num=fid_num)

    def __is_same_si(self, si1: List[StrategyInsight], si2: List[StrategyInsight]) -> bool:
        if len(si1) != len(si2):
            return False

        si1_sorted = sorted(si1, key=lambda x: x.symbol_id)
        si2_sorted = sorted(si2, key=lambda x: x.symbol_id)
        for a, b in zip(si1_sorted, si2_sorted):
            if a != b:
                return False
        return True



from typing import List, Dict, Tuple, Union, Any
import threading
import copy

class TimelyCache_Snapshot(object):
    """! Keep a limited list of latest snapshot
    self.__dict: it stores a pair of key and value, the value is the latest snapshot
    self.__time_index: it stores a list of key in time sequential. Key value can be duplicated
    """
    def __init__(self, max_size: int = 1000, eq_fun: Any = None):
        self.__dict: Dict[Any, Any] = dict()
        self.__time_index: List[Any] = list()
        self.__hist_data: List[Union[Any, Any]] = list()
        self.__mutex: threading.Lock = threading.Lock()
        self.__max_size = max_size
        self.__eq_fun = eq_fun

    def upsert_multi(self, keys: List[Any], values: List[Any]) -> List[bool]:
        is_new_list: List[bool] = list()
        with self.__mutex:
            for key, value in zip(keys, values):
                is_new_list.append(self.__upsert_ele_imp(key=key, value=value))

            return is_new_list

        is_new_list = [False] * len(keys)
        return is_new_list

    def upsert_ele(self, key: Any, value: Any) -> bool:
        with self.__mutex:
            return self.__upsert_ele_imp(key=key, value=value)

    def is_exist(self, key: Any) -> bool:
        with self.__mutex:
            return key in self.__dict

    def get_records_by_index(self, idx: int) -> Any:
        with self.__mutex:
            return copy.deepcopy(self.__dict[self.__time_index[idx]])

    def get_last_n_records_in_time_series(self, n: int) -> List[Any]:
        with self.__mutex:
            return [ele[1] for ele in self.__hist_data[-n:]]

    def get_records_in_time_series(self) -> List[Any]:
        with self.__mutex:
            return [ele[1] for ele in self.__hist_data]

    def get_ele(self, key: Any) -> Any:
        with self.__mutex:
            if key not in self.__dict:
                return None
            return copy.deepcopy(self.__dict[key])

    def size(self) -> int:
        with self.__mutex:
            return len(self.__dict)

    def items(self):
        with self.__mutex:
            for k, v in self.__dict.items():
                yield k, v

    def __upsert_ele_imp(self, key: Any, value: Any) -> bool:
        is_new = False
        if key not in self.__dict:
            # new value found
            self.__time_index.append(key)
            is_new = True

            # save record
            self.__hist_data.append((key, value))
            self.__dict[key] = value
        else:
            if (None is not self.__eq_fun and self.__eq_fun(first=self.__dict[key], second=value))\
               or self.__dict[key] == value:
                # save value found
                return is_new
            else:
                # new value found
                is_new = True

                # save record
                self.__hist_data.append((key, value))
                self.__dict[key] = value

        self.__remove_extra_records()
        return is_new

    def __remove_extra_records(self):
        if len(self.__dict) > self.__max_size:
            total_old_records = len(self.__dict) - self.__max_size
            old_key_list: List[Any] = list()
            idx = 0
            while total_old_records > len(old_key_list):
                if len(old_key_list) == 0:
                    old_key_list.append(self.__time_index[idx])
                else:
                    if old_key_list[-1] != self.__time_index[idx]:
                        old_key_list.append(self.__time_index[idx])

                while old_key_list[-1] == self.__time_index[idx]:
                    idx += 1

            last_n = idx
            # remove from dict
            for old_key in old_key_list:
                del self.__dict[old_key]
            # remove from list
            del self.__time_index[:last_n]
            # remove from hist data
            del self.__hist_data[:total_old_records]

class TimelyCache_Hist(object):
    """! Keep a limited list of latest records
    self.__dict: it stores a pair of key and value
    self.__time_index: it stores a list of key in time sequential
    """
    def __init__(self, max_size: int = 1000, eq_fun: Any = None):
        self.__dict: Dict[Any, Any] = dict()
        self.__time_index: List[Union[Any, Any]] = list()
        self.__mutex: threading.Lock = threading.Lock()
        self.__max_size = max_size
        self.__eq_fun = eq_fun

    def upsert_multi(self, keys: List[Any], values: List[Any]) -> List[bool]:
        is_new_list: List[bool] = list()
        with self.__mutex:
            for key, value in zip(keys, values):
                is_new_list.append(self.__upsert_ele_imp(key=key, value=value))

            return is_new_list

        is_new_list = [False] * len(keys)
        return is_new_list

    def upsert_ele(self, key: Any, value: Any) -> bool:
        with self.__mutex:
            return self.__upsert_ele_imp(key=key, value=value)

    def is_exist(self, key: Any) -> bool:
        with self.__mutex:
            return key in self.__dict

    def get_records_by_index(self, idx: int) -> Any:
        with self.__mutex:
            return copy.deepcopy(self.__time_index[idx][1])

    def get_last_n_records_in_time_series(self, n: int) -> List[Any]:
        with self.__mutex:
            return [ele[1] for ele in self.__time_index[-n:]]

    def get_records_in_time_series(self) -> List[Any]:
        with self.__mutex:
            return [ele[1] for ele in self.__time_index]

    def get_ele(self, key: Any) -> Any:
        with self.__mutex:
            if key not in self.__dict:
                return None
            return copy.deepcopy(self.__dict[key])

    def size(self) -> int:
        with self.__mutex:
            return len(self.__dict)

    def items(self):
        with self.__mutex:
            for k, v in self.__dict.items():
                yield k, v

    def __upsert_ele_imp(self, key: Any, value: Any) -> bool:
        is_new = False
        if key not in self.__dict:
            # found new record
            is_new = True

            self.__time_index.append((key, value))
            self.__dict[key] = value
        else:
            if (None is not self.__eq_fun and self.__eq_fun(first=self.__dict[key], second=value))\
               or self.__dict[key] == value:
                # found old record
                is_new = False
            else:
                # found new record
                is_new = True

                self.__time_index.append((key, value))
                self.__dict[key] = value

        self.__remove_extra_records()

        return is_new

    def __remove_extra_records(self):
        if len(self.__dict) > self.__max_size:
            extra_n = len(self.__dict) - self.__max_size
            for ele in self.__time_index[:extra_n]:
                del self.__dict[ele[0]] # ele = (old_key, old_value)
            del self.__time_index[:extra_n]
